/*ParseAtomHLike parse information from the atom h-like command line */
#include "cddefines.h"
#include "taulines.h"
#include "hydrogenic.h"
#include "dampon.h"
#include "physok.h"
#include "iso.h"
#include "rfield.h"
#include "thlo.h"
#include "parse.h"

void ParseAtomHLike(char *chCard )
{
	int lgEOL;
	long int i , 
		nh ,
		ipZ;

#	ifdef DEBUG_FUN
	fputs( "<+>ParseAtomHLike()\n", debug_fp );
#	endif

	if( lgMatch("COLL",chCard) )
	{
		/* option to turn hydrogen collisions off, all are
		 * set to 1 in zero.  command can accept only one option at a time */
		if( lgMatch("EXCI",chCard) )
		{
			/* turn off collisional excitation */
			iso.lgColl_excite[ipHYDROGEN] = FALSE;
			physok.lgPhysOK = FALSE;
		}
		else if( lgMatch("IONI",chCard) )
		{
			/* turn off collisional ionization */
			iso.lgColl_ionize[ipHYDROGEN] = FALSE;
			physok.lgPhysOK = FALSE;
		}

		else if( lgMatch("2S2P",chCard) || lgMatch("2P2S",chCard) )
		{
			/* >>chng 02 feb 07, change from 2s2p to l-mixing */
			/* this is the atom h-like collision l-mixing command */
			fprintf(ioQQQ,"This command changed to ATOM H-LIKE COLLISIONS L-MIXING\n");
			fprintf(ioQQQ,"I will parse it for now, but may not in the future.\n");
			/* turn off 2s - 2p collisions */
			iso.lgColl_l_mixing[ipHYDROGEN] = FALSE;
			physok.lgPhysOK = FALSE;
		}

		else if( lgMatch("L-MI",chCard)  )
		{
			/* >>chng 02 feb 07, change from 2s2p to l-mixing */
			/* this is the atom h-like collision l-mixing command */
			iso.lgColl_l_mixing[ipHYDROGEN] = FALSE;
			physok.lgPhysOK = FALSE;
		}
		else
		{
			/* turn everything off, since no keyword given */
			iso.lgColl_excite[ipHYDROGEN] = FALSE;
			iso.lgColl_ionize[ipHYDROGEN] = FALSE;
			iso.lgColl_l_mixing[ipHYDROGEN] = FALSE;
			physok.lgPhysOK = FALSE;
		}
	}

	else if( lgMatch("DAMP",chCard) )
	{
		/* turn off absorption due to lyalpha damping wings */
		dampon.DampOnFac = 0.;
	}

	else if( lgMatch("EMIS",chCard) )
	{
		/* hydrogenic emission lines will be emissivity not intensity or luminosity */
		hydro.lgHydEmiss = TRUE;
	}

	else if( lgMatch("LEVE",chCard) )
	{
		/* number of levels for hydrogen at, 2s is this plus one */
		/* there are two options here,
		 * when keyword ELEMENT appears, scan off element name and change levels only for
		 * that one.
		 * when there is no ELEMENT then set all nhlevels to same number */

		/* lgHydroMalloc is FALSE at start of calculation, set true when space 
		 * allocated for the hydrogen lines.  Once done we must ignore all 
		 * future changes in the number of levels */
		if( !lgHydroMalloc )
		{
			i = 5;
			nh = (long int)FFmtRead(chCard,&i,LINELENGTH,&lgEOL);

			if( lgEOL )
			{
				/* must be a number, or a key, either large (or limit) or small
				 * if no number but LARGER or SMALL then set to preset number */
				if( lgMatch("LARG",chCard) )
				{
					/* there is no real limit, but this seems pretty big */
					nh = 50;
				}

				/* this is small or compact keyword */
				else if( lgMatch("SMAL",chCard) || lgMatch("COMP",chCard) )
				{
					/* a small atom is the same as c84 */
					nh = 10;
				}

				/* this is limit keyword, sets to largest possible value */
				else if( lgMatch("LIMI",chCard) )
				{
					nh = NHYDRO_MAX_LEVEL-2;
				}

				else
				{
					/* punch out if no number */
					NoNumb(chCard);
				}

			}

			if( nh < 4 )
			{
				fprintf( ioQQQ, " This would be too few levels.\n" );
				puts( "[Stop in ParseAtomHLike]" );
				cdEXIT(1);
			}

			if( nh < iso.nTopOff[ipHYDROGEN] )
			{
				fprintf( ioQQQ, " Not possible to set nhlvl to less than top off=%4li\n", 
				  iso.nTopOff[ipHYDROGEN] );
				fprintf( ioQQQ, " change nHTopOff first with ATOM H-LIKE TOPOFF command\n");
				puts( "[Stop in ParseAtomHLike]" );
				cdEXIT(1);
			}

			if( nh > NHYDRO_MAX_LEVEL-2 )
			{
				fprintf( ioQQQ, " Not possible to set nhlvl to >NHYDRO_MAX_LEVEL-2= %li\n", 
				  NHYDRO_MAX_LEVEL-2 );
				fprintf( ioQQQ, " change NHYDRO_MAX_LEVEL\n");
				puts( "[Stop in ParseAtomHLike]" );
				cdEXIT(1);
			}

			/* this is check that alpha transition of highest level is within energy
			 * bounds of continuum */
			if( 2. / POW3((double)nh) < rfield.emm )
			{
				fprintf( ioQQQ, " Not possible to set iso.nLevels[ipHYDROGEN][ipHYDROGEN] to such a high value, since "
					"alpha transition not within energy bounds of code\n");

				fprintf( ioQQQ, " lowest energy is %e and corresponding highest level is %li\n" ,
					rfield.emm, (long)pow(2./rfield.emm, 0.3333) );
				puts( "[Stop in ParseAtomHLike]" );
				cdEXIT(1);
			}

			/* we now have the desired number of levels, and it has been fully qualified. 
			 * now check whether ELEMENT keyword is on line, if not then set all to
			 * the number, if so then only element in use */
			if( lgMatch("ELEM",chCard) )
			{
				/* check which element is on the line, ipZ is atomic number on C scale */
				ipZ = GetElem(chCard);
				/* only set number of levels for this one element */
				iso.nHighest[ipHYDROGEN][ipZ] = nh;
				/* >>chng 02 jan 19, increase numb of H levels by 1 so that loops are < n 
				 * should be no net change in levels as a result */
				iso.nLevels[ipHYDROGEN][ipZ] = nh+1;
				iso.nPrintLevels[ipHYDROGEN][ipZ] = nh;
			}
			else
			{
				/* keyword element did not appear, so set all h-like species to this number */
				for( i=0; i<LIMELM; ++i )
				{
					iso.nHighest[ipHYDROGEN][i] = nh;
					iso.nLevels[ipHYDROGEN][i] = nh+1;
					iso.nPrintLevels[ipHYDROGEN][i] = nh-1;
				}
			}
		}
	}

	else if( lgMatch("LOWE",chCard) )
	{
		/* set lowest temp for departure coef, default is 1000 */
		i = 5;
		thlo.HydTempLimit = (float)FFmtRead(chCard,&i,LINELENGTH,&lgEOL);
		if( thlo.HydTempLimit <= 10. )
			thlo.HydTempLimit = (float)pow(10.f,thlo.HydTempLimit);
	}

	else if( lgMatch("REDI",chCard) )
	{
		int ipRedis=0;
		/* there are three functions, PRD_, CRD_, and CRDW,
		 * representing partial redistribution, 
		 * complete redistribution (doppler core only, no wings)
		 * and complete with wings */
		/* partial redistribution */
		if( lgMatch(" PRD",chCard) )
		{
			ipRedis = ipPRD;
		}
		/* complete redistribution no wings */
		else if( lgMatch(" CRD",chCard) )
		{
			ipRedis = ipCRD;
		}
		/* complete redistribution with wings */
		else if( lgMatch("CRDW",chCard) )
		{
			ipRedis = ipCRDW;
		}

		/* if not SHOW option (handled below) then we have a problem */
		else if( !lgMatch("SHOW",chCard) )
		{
			fprintf(ioQQQ," There should have been a second keyword on this command.\n");
			fprintf(ioQQQ," Options are _PRD, _CRD, CRDW (_ is space).  Sorry.\n");
			puts( "[Stop in ParseAtomHLike]" );
			cdEXIT(1);
		}

		/* resonance lines - not Lya*/
		if( lgMatch("ALPH",chCard) )
		{
			hydro.ipLya = ipRedis;
		}
		/* Lya itself */
		if( lgMatch("RESO",chCard) )
		{
			hydro.ipReso = ipRedis;
		}
		/* subordinate lines */
		else if( lgMatch("SUBO",chCard) )
		{
			hydro.ipSub = ipRedis;
		}
		/* the show option, say what we are assuming */
		else if( lgMatch("SHOW",chCard) )
		{
			fprintf(ioQQQ," Ly a is ");
			if( hydro.ipLya ==ipCRDW )
			{
				fprintf(ioQQQ,"complete redistribution with wings\n");
			}
			else if( hydro.ipLya ==ipCRD )
			{
				fprintf(ioQQQ,"complete redistribution with core only.\n");
			}
			else if( hydro.ipLya ==ipPRD )
			{
				fprintf(ioQQQ,"partial redistribution.\n");
			}
			else
			{
				fprintf(ioQQQ," Impossible value for hydro.ipLya.\n");
				insane();
				ShowMe();
				puts( "[Stop in ParseAtomFeII]" );
				cdEXIT(1);
			}

			fprintf(ioQQQ," Other H resonance lines are ");
			if( hydro.ipReso ==ipCRDW )
			{
				fprintf(ioQQQ,"complete redistribution with wings\n");
			}
			else if( hydro.ipReso ==ipCRD )
			{
				fprintf(ioQQQ,"complete redistribution with core only.\n");
			}
			else if( hydro.ipReso ==ipPRD )
			{
				fprintf(ioQQQ,"partial redistribution.\n");
			}
			else
			{
				fprintf(ioQQQ," Impossible value for hydro.ipReso.\n");
				insane();
				ShowMe();
				puts( "[Stop in ParseAtomFeII]" );
				cdEXIT(1);
			}

			fprintf(ioQQQ," H subordinate lines are ");
			if( hydro.ipSub ==ipCRDW )
			{
				fprintf(ioQQQ,"complete redistribution with wings\n");
			}
			else if( hydro.ipSub ==ipCRD )
			{
				fprintf(ioQQQ,"complete redistribution with core only.\n");
			}
			else if( hydro.ipSub ==ipPRD )
			{
				fprintf(ioQQQ,"partial redistribution.\n");
			}
			else
			{
				fprintf(ioQQQ," Impossible value for hydro.ipSub.\n");
				insane();
				ShowMe();
				puts( "[Stop in ParseAtomHLike]" );
				cdEXIT(1);
			}
		}
		else
		{
			fprintf(ioQQQ," here should have been another keyword on this command.\n");
			fprintf(ioQQQ," Options are ALPHA, RESONANCE, SUBORDINATE.  Sorry.\n");
			puts( "[Stop in ParseAtomHLike]" );
			cdEXIT(1);
		}
	}

	else if( lgMatch("TOPO",chCard) )
	{
		/* set lowest level for hydrogen atom top off topoff */
		i = 5;
		iso.nTopOff[ipHYDROGEN] = (long int)FFmtRead(chCard,&i,LINELENGTH,&lgEOL);

		if( iso.nTopOff[ipHYDROGEN] > iso.nLevels[ipHYDROGEN][ipHYDROGEN] )
		{
			fprintf( ioQQQ, " Not possible to set nHTopOff to more than nhlvl=%4ld\n", 
			  iso.nLevels[ipHYDROGEN][ipHYDROGEN] );
			fprintf( ioQQQ, " change nhlvl first with ATOM H-LIKE LEVELS command\n");
			puts( "[Stop in ParseAtomHLike]" );
			cdEXIT(1);
		}

		if( lgEOL )
		{
			NoNumb(chCard);
		}

		if( lgMatch(" ADD",chCard) )
		{
			/* add extra part */
			strcpy( hydro.chHTopType, " add" );
		}
		else if( lgMatch("SCAL",chCard) )
		{
			/* use scale factors */
			strcpy( hydro.chHTopType, "scal" );
		}
		else
		{
			/* this is the default */
			strcpy( hydro.chHTopType, "scal" );
		}
	}

	/* which type of matrix solution, populations, lowt, departure coef */
	else if( lgMatch("MATR",chCard) )
	{
		if( lgMatch("POPU",chCard) )
		{
			strcpy( hydro.chTypeAtom , "POPU" );
		}
		else if( lgMatch("LOWT",chCard) )
		{
			strcpy( hydro.chTypeAtom , "LOWT" );
		}
		else
		{
			fprintf( ioQQQ, " There should have been a keyword on the MATRIX option.\n");
			fprintf( ioQQQ, " The keywords are POPUlations, LOWTe, and DEPArture coef.\n");
			fprintf( ioQQQ, " Stop in ParseAtomHLike\n" );
			puts( "[Stop in ParseAtomHLike]" );
			cdEXIT(1);
		}
	}

	else
	{
		fprintf( ioQQQ, " There should have been a keyword - Stop in ParseAtomHLike\n" );
		puts( "[Stop in ParseAtomHLike]" );
		cdEXIT(1);
	}

#	ifdef DEBUG_FUN
	fputs( " <->ParseAtomHLike()\n", debug_fp );
#	endif
	return;
}

